# Copyright 2021-2022 BlueCat Networks (USA) Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
"""Constants for versions of BlueCat Address Manager."""

from bluecat_libraries.address_manager.api._version import Version

V9_1_0 = Version("9.1.0")
V9_2_0 = Version("9.2.0")
V9_3_0 = Version("9.3.0")
V9_4_0 = Version("9.4.0")
