# Copyright 2020 BlueCat Networks (USA) Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
"""Package for exposing a client with autogenerated methods for working with BlueCat Address Manager's API."""

from bluecat_libraries.address_manager.api.rest.auto.client import Client

__all__ = [
    "Client",
]
