# Copyright 2021 BlueCat Networks (USA) Inc. and its affiliates.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
"""Modules for working with BlueCat Failover API."""
from bluecat_libraries.address_manager.failover_api.client import Client

__all__ = ["Client"]
